/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.VariableOperand;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author A. Lefebvre
 */
public abstract class BasicOperator extends BasicExpression implements Operator {

	protected VariableOperand result = null;
	protected boolean verified = false;
	protected Expression[] expressions;

	public BasicOperator() {
	}

	public BasicOperator(Expression[] expressions) {
		this.expressions = expressions;
	}

	public BasicOperator(PType type) {
		super(type);
	}

	public BasicOperator(PType type, Expression[] expressions) {
		super(type);
		this.expressions = expressions;
	}

	public Object clone(Object clone, java.util.Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicOperator bo = (BasicOperator) clone;
		bo.result = (VariableOperand) getClone(result, obj2clone);
		bo.verified = verified;
		if (expressions != null) {
			bo.expressions = new Expression[expressions.length];
			//when cloning an array, the elements are not cloned
			for(int i=0; i<expressions.length; i++) {
				if (expressions[i] != null) {
					bo.expressions[i] =(Expression) getClone(expressions[i], obj2clone);
				}
			}
		}
		return clone;
	}

	// IMPLEMENTATION OF THE Expression INTERFACE //
	//--------------------------------------------//

	public Expression getExpression(int i) {
		if (expressions == null) {
			logger.log(BasicLevel.FATAL, "Operand of the operaor "
				+ getOperatorString(), new NullPointerException());
		}
		return expressions[i];
	}

	public synchronized void setExpression(int idx, Expression e) {
		if (expressions == null) {
			expressions = new Expression[idx + 1];
			logger.log(BasicLevel.WARN,
				"Allocation of the operand array (" + idx + ") for the operator "
				+ getOperatorString());
		} else if (idx >= expressions.length){
			Expression[] neo = new Expression[idx + 1];
			System.arraycopy(expressions, 0, neo, 0, expressions.length);
			expressions = neo;
			logger.log(BasicLevel.WARN,
				"Reallocation " + expressions.length + " to " + idx
				+ "of the operand array for the operator "
				+ getOperatorString());
		}
		expressions[idx] = e;
	}

	public int getOperandNumber() {
		return expressions.length;
	}
}
