/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryLogicalOperator;
import org.objectweb.medor.expression.api.Expression;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicUnaryLogicalOperator
        extends BasicUnaryOperator
        implements UnaryLogicalOperator {

	public BasicUnaryLogicalOperator() {
		super(PTypeSpace.BOOLEAN);
	}

	public BasicUnaryLogicalOperator(Expression e) {
		super(PTypeSpace.BOOLEAN, e);
	}
	
    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public org.objectweb.medor.expression.api.Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        try {
            result.setValue(expressions[0].evaluate(pos, o).getBoolean());
        } catch (NullPointerException e) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
		return result;
    }

    public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
        if (expressions[0] != null) {
            expressions[0].compileExpression();
            if (expressions[0].getType().getTypeCode() ==
				PType.TYPECODE_BOOLEAN) {
                result = new BasicVariableOperand(type);
                verified = true;
            } else
			    //it is not a boolean type
                throw new TypingException("Attempt a boolean type");
        } else
		    //Operands not yet assigned
            throw new MalformedExpressionException("Null children value");
		return result;
    }

    // IMPLEMENTATION OF THE UnaryLogicalOperator INTERFACE //
    //-------------------------------------------------------//

    public abstract boolean evaluate(boolean boole);

}
