/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.VariableOperand;

import java.util.Date;

public class BasicVariableOperand
	extends BasicOperand
	implements VariableOperand {

    public BasicVariableOperand() {
        super();
    }
    public BasicVariableOperand(BasicOperand p) {
        super(p);
    }
    public BasicVariableOperand(PType p) {
        super(p);
    }

    public BasicVariableOperand(boolean b) {
        super(b);
    }

    public BasicVariableOperand(char p) {
        super(p);
    }

    public BasicVariableOperand(byte p) {
        super(p);
    }

    public BasicVariableOperand(short p) {
        super(p);
    }

    public BasicVariableOperand(int p) {
        super(p);
    }

    public BasicVariableOperand(long p) {
        super(p);
    }

    public BasicVariableOperand(float p) {
        super(p);
    }

    public BasicVariableOperand(double p) {
        super(p);
    }

    public BasicVariableOperand(String p) {
        super(p);
    }

    public BasicVariableOperand(Date p) {
        super(p);
    }

    public void setType(PType type) {
        longValue = 0;
        doubleValue = 0;
        objectValue = null;
        this.type = type;
    }

    public void setValue(boolean p) throws TypingException {
        if (PTypeSpace.BOOLEAN.isa(type)) {
            longValue = (p ? 1 : 0);
            doubleValue = (p ? 1 : 0);
        } else
            throw new TypingException("This Operand can not store a boolean value");
    }

    public void setValue(int p) throws TypingException {
        if (PTypeSpace.INT.isa(type)) {
            longValue = p;
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store an integer value " + type.getTypeCode());
    }

    public void setValue(byte p) throws TypingException {
        if (PTypeSpace.BYTE.isa(type)) {
            longValue = p;
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store a byte value");
    }


    public void setValue(short p) throws TypingException {
        if (PTypeSpace.SHORT.isa(type)) {
            longValue = p;
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store a short value");
    }


    public void setValue(long p) throws TypingException {
        if (PTypeSpace.LONG.isa(type)) {
            longValue = p;
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store a long value");
    }

    public void setValue(float p) throws TypingException {
        if (PTypeSpace.FLOAT.isa(type)) {
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store a float value");
    }

    public void setValue(double p) throws TypingException {
        if (PTypeSpace.DOUBLE.isa(type)) {
            doubleValue = p;
        } else
            throw new TypingException("This Operand can not store a double value");
    }

    public void setValue(char p) throws TypingException {
        if (PTypeSpace.CHAR.isa(type)) {
            doubleValue = p;
            longValue = p;
        } else
            throw new TypingException("This Operand can not store a CHAR value");
    }

    public void setValue(String p) throws TypingException {
        if (PTypeSpace.STRING.isa(type)) {
            objectValue = p;
        } else
            throw new TypingException("This Operand can not store a string value " + type.getJavaName());
    }

    public void setValue(Date p) throws TypingException {
        objectValue = p;
    }


    public void setValue(Object p) throws TypingException {
        objectValue = p;
        switch(type.getTypeCode()) {
        case PType.TYPECODE_BOOLEAN:
            longValue = ((Boolean) p).booleanValue() ? 1 : 0;
            doubleValue = longValue;
            break;
        case PType.TYPECODE_BYTE:
            longValue = ((Byte) p).byteValue();
            doubleValue = longValue;
            break;
        case PType.TYPECODE_CHAR:
            longValue = ((Character) p).charValue();
            doubleValue = longValue;
            break;
        case PType.TYPECODE_SHORT:
            longValue = ((Short) p).shortValue();
            doubleValue = longValue;
            break;
        case PType.TYPECODE_INT:
            longValue = ((Integer) p).intValue();
            doubleValue = longValue;
            break;
        case PType.TYPECODE_LONG:
            longValue = ((Long) p).longValue();
            doubleValue = longValue;
            break;
        case PType.TYPECODE_FLOAT:
            doubleValue = ((Float) p).floatValue();
            break;
        case PType.TYPECODE_DOUBLE:
            doubleValue = ((Double) p).doubleValue();
            break;
        }
    }
}
