/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.medor.expression.parser.lib;

import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.expression.lib.Concat;
import org.objectweb.medor.expression.lib.TypeConverter;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PType;

/**
 * This class is a rewriter of Expression, replacing the plus operator between
 * string to the concat operator.
 *
 * @author S.Chassande-Barrioz
 */
public class ReplaceStringPlusByConcat {


	/**
	 * replaces the plus operator between string to the concat operator.
	 * @param e is the expression to rewrite
	 * @return the new expression
	 */
	public Expression rewrite(Expression e) throws MalformedExpressionException {
		if (e instanceof Operator) {
			Operator op = (Operator) e;
			for(int i=op.getOperandNumber()-1; i>=0; i--) {
				op.setExpression(i, rewrite(op.getExpression(i)));
			}
			if (op instanceof Plus) {
				Expression left = op.getExpression(0);
				Expression right = op.getExpression(1);
				if (left.getType() == PTypeSpace.STRING
					|| right.getType() == PTypeSpace.STRING) {
					return new Concat(
						convertToString(left),
						convertToString(right));
				}
			}
		}
		return e;
	}

	/**
	 * Converts an expression to a string.
	 */
	private Expression convertToString(Expression e) throws MalformedExpressionException {
		return (e.getType() == PTypeSpace.STRING
			? e
			: new TypeConverter(e, PTypeSpace.STRING));
	}

}
